@@ -72,7 +72,8 @@ static void	xtm_process_window_finalize			(GObject *object);
 static void	xtm_process_window_hide				(GtkWidget *widget);
 
 static void	emit_destroy_signal				(XtmProcessWindow *window);
-static gboolean	emit_delete_event_signal			(XtmProcessWindow *window, GdkEvent *event);
+static gboolean	xtm_process_window_configure_event		(XtmProcessWindow *window, GdkEvent *event);
+static gboolean	xtm_process_vpaned_move_event			(XtmProcessWindow *window, GdkEventButton *event);
 static gboolean xtm_process_window_key_pressed	(XtmProcessWindow *window, GdkEventKey *event);
 static void	toolbar_update_style				(XtmProcessWindow *window);
 static void	monitor_update_step_size			(XtmProcessWindow *window);
@@ -212,6 +213,17 @@ xtm_process_window_class_init (XtmProcessWindowClass *klass)
 	widget_class->hide = xtm_process_window_hide;
 }
 
+static void
+xtm_show_legend (XtmProcessWindow *window)
+{
+	gboolean show_legend;
+
+	g_object_get (window->settings,
+				  "show-legend", &show_legend,
+				  NULL);
+	gtk_widget_set_visible (GTK_WIDGET (gtk_builder_get_object (window->builder, "legend")), show_legend);
+}
+
 static void
 xtm_process_window_init (XtmProcessWindow *window)
 {
@@ -219,7 +231,7 @@ xtm_process_window_init (XtmProcessWindow *window)
 	GtkWidget *icon;
 	GtkToolItem *xwininfo;
 	gint width, height;
-	gchar *markup;
+	gboolean show_legend;
 
 	window->settings = xtm_settings_get_default ();
 
@@ -231,8 +243,9 @@ xtm_process_window_init (XtmProcessWindow *window)
 	if (width >= 1 && height >= 1)
 		gtk_window_resize (GTK_WINDOW (window->window), width, height);
 	g_signal_connect_swapped (window->window, "destroy", G_CALLBACK (emit_destroy_signal), window);
-	g_signal_connect_swapped (window->window, "delete-event", G_CALLBACK (emit_delete_event_signal), window);
 	g_signal_connect_swapped (window->window, "key-press-event", G_CALLBACK(xtm_process_window_key_pressed), window);
+	g_signal_connect_swapped(window->window, "configure-event",
+	    G_CALLBACK(xtm_process_window_configure_event), window);
 
 	window->toolbar = GTK_WIDGET (gtk_builder_get_object (window->builder, "process-toolbar"));
 	g_signal_connect_swapped (window->settings, "notify::toolbar-style", G_CALLBACK (toolbar_update_style), window);
@@ -243,6 +256,8 @@ xtm_process_window_init (XtmProcessWindow *window)
 
 	window->settings_button = xtm_settings_tool_button_new ();
 	gtk_toolbar_insert (GTK_TOOLBAR (window->toolbar), GTK_TOOL_ITEM (window->settings_button), 1);
+	g_signal_connect_swapped (window->settings, "notify::show-legend", G_CALLBACK (xtm_show_legend), window);
+	g_object_notify (G_OBJECT (window->settings), "show-legend");
 
 	icon = gtk_image_new_from_icon_name ("xc_crosshair", GTK_ICON_SIZE_LARGE_TOOLBAR);
 	xwininfo = gtk_tool_button_new (icon, _("Identify Window"));
@@ -268,6 +283,8 @@ xtm_process_window_init (XtmProcessWindow *window)
 		window->vpaned = GTK_WIDGET (gtk_builder_get_object (window->builder, "mainview-vpaned"));
 		if (handle_position > -1)
 			gtk_paned_set_position (GTK_PANED (window->vpaned), handle_position);
+		g_signal_connect_swapped(window->vpaned, "button-release-event",
+		    G_CALLBACK(xtm_process_vpaned_move_event), window);
 
 		toolitem = GTK_WIDGET (gtk_builder_get_object (window->builder, "graph-cpu"));
 		window->cpu_monitor = xtm_process_monitor_new ();
@@ -312,22 +329,19 @@ xtm_process_window_init (XtmProcessWindow *window)
 
 	window->treeview = xtm_process_tree_view_new ();
 	gtk_widget_show (window->treeview);
-	markup = g_strdup_printf (_("<span foreground='#000000' background='#aed581'>    </span> Starting task\n"
-								"<span foreground='#000000' background='#fff176'>    </span> Changing task\n"
-								"<span foreground='#000000' background='#e57373'>    </span> Terminating task"));
-	gtk_widget_set_tooltip_markup (GTK_WIDGET (gtk_builder_get_object (window->builder, "scrolledwindow")), markup);
-	g_free (markup);
 	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (window->builder, "scrolledwindow")), window->treeview);
 
+	g_object_get (window->settings,
+				  "show-legend", &show_legend,
+				  NULL);
+	gtk_widget_set_visible (GTK_WIDGET (gtk_builder_get_object (window->builder, "legend")), show_legend);
+
 	window->filter_entry = GTK_WIDGET(gtk_builder_get_object (window->builder, "filter-entry"));
 	g_signal_connect (G_OBJECT(window->filter_entry), "icon-press", G_CALLBACK(filter_entry_icon_pressed_cb), NULL);
 	g_signal_connect (G_OBJECT(window->filter_entry), "changed", G_CALLBACK(filter_entry_keyrelease_handler), window->treeview);
 	gtk_widget_set_tooltip_text (window->filter_entry, _("Filter on process name"));
 
 	gtk_widget_grab_focus (GTK_WIDGET (window->filter_entry));
-
-	g_object_unref (window->builder);
-	window->builder = NULL;
 }
 
 static void
@@ -349,6 +363,9 @@ xtm_process_window_finalize (GObject *object)
 
 	if (XTM_IS_SETTINGS (window->settings))
 		g_object_unref (window->settings);
+
+	g_object_unref (window->builder);
+	window->builder = NULL;
 }
 
 /**
@@ -362,28 +379,33 @@ emit_destroy_signal (XtmProcessWindow *window)
 }
 
 static gboolean
-emit_delete_event_signal (XtmProcessWindow *window, GdkEvent *event)
-{
-	gboolean ret;
-	gint width, height, handle_position;
-	guint sort_column_id;
-	GtkSortType sort_type;
+xtm_process_window_configure_event(XtmProcessWindow *window,
+    GdkEvent *event) {
 
-	gtk_window_get_size (GTK_WINDOW (window->window), &width, &height);
-	xtm_process_tree_view_get_sort_column_id (XTM_PROCESS_TREE_VIEW (window->treeview), (gint*)&sort_column_id, &sort_type);
+	if (NULL != window &&
+	    NULL != event && GDK_CONFIGURE == event->configure.type) {
+		g_object_set (window->settings,
+		    "window-width", event->configure.width,
+		    "window-height", event->configure.height,
+		    NULL);
+	}
 
-	handle_position = gtk_paned_get_position (GTK_PANED (window->vpaned));
+	return (FALSE);
+}
 
-	g_object_set (window->settings,
-				  "window-width", width,
-				  "window-height", height,
-				  "sort-column-id", sort_column_id,
-				  "sort-type", sort_type,
-				  "handle-position", handle_position,
-				  NULL);
+static gboolean
+xtm_process_vpaned_move_event(XtmProcessWindow *window,
+    GdkEventButton *event __unused) {
+	gint handle_position;
 
-	g_signal_emit_by_name (window, "delete-event", event, &ret, G_TYPE_BOOLEAN);
-	return ret;
+	if (NULL != window) {
+		handle_position = gtk_paned_get_position(GTK_PANED(window->vpaned));
+		g_object_set (window->settings,
+		    "handle-position", handle_position,
+		    NULL);
+	}
+
+	return (FALSE);
 }
 
 static gboolean
@@ -393,7 +415,8 @@ xtm_process_window_key_pressed (XtmProcessWindow *window, GdkEventKey *event)
 
 	if (event->keyval == GDK_KEY_Escape ||
 		(event->keyval == GDK_KEY_q && (event->state & GDK_CONTROL_MASK))) {
-		emit_delete_event_signal (window, (GdkEvent*) event);
+		xtm_settings_save_settings (window->settings);
+		g_signal_emit_by_name (window, "delete-event", event, &ret, G_TYPE_BOOLEAN);
 		ret = TRUE;
 	}
 	else if (event->keyval == GDK_KEY_f && (event->state & GDK_CONTROL_MASK)) {
@@ -462,6 +485,7 @@ static void
 show_about_dialog (XtmProcessWindow *window)
 {
 	const gchar *authors[] = {
+		"(c) 2018-2019 Rozhuk Ivan",
 		"(c) 2014 Landry Breuil",
 		"(c) 2014 Harald Judt",
 		"(c) 2014 Peter de Ridder",
@@ -470,6 +494,7 @@ show_about_dialog (XtmProcessWindow *window)
 		"(c) 2005-2008 Johannes Zellner",
 		"",
 		"FreeBSD",
+		"  \342\200\242 Rozhuk Ivan",
 		"  \342\200\242 Mike Massonnet",
 		"  \342\200\242 Oliver Lehmann",
 		"",
@@ -496,7 +521,7 @@ show_about_dialog (XtmProcessWindow *window)
 	gtk_show_about_dialog (GTK_WINDOW (window->window),
 		"program-name", _("Task Manager"),
 		"version", PACKAGE_VERSION,
-		"copyright", "Copyright \302\251 2005-2014 The Xfce development team",
+		"copyright", "Copyright \302\251 2005-2019 The Xfce development team",
 		"logo-icon-name", "utilities-system-monitor",
 #if !GTK_CHECK_VERSION(3, 0, 0)
 		"icon-name", GTK_STOCK_ABOUT,
