@@ -18,9 +18,7 @@
 
 #include <stdlib.h>
 #include <err.h>
-/* for getpwuid() */
 #include <sys/types.h>
-#include <pwd.h>
 /* for sysctl() */
 #include <sys/param.h>
 #include <sys/sched.h>
@@ -54,7 +52,6 @@ gboolean get_task_list (GArray *task_list)
 	struct kinfo_proc2 *kp;
 #endif
 	Task t;
-	struct passwd *passwdp;
 	char **args;
 	gchar* buf;
 	int nproc, i;
@@ -102,6 +99,7 @@ gboolean get_task_list (GArray *task_list)
 #else
 		struct kinfo_proc2 p = kp[i];
 #endif
+		memset(&t, 0, sizeof(t));
 		t.pid = p.p_pid;
 		t.ppid = p.p_ppid;
 		t.uid = p.p_uid;
@@ -113,12 +111,14 @@ gboolean get_task_list (GArray *task_list)
 		g_strlcpy(t.name, p.p_comm, strlen(p.p_comm) + 1);
 		/* shamelessly stolen from top/machine.c */
 		if (!P_ZOMBIE(&p)) {
-			size = 128;
+			size = 1024;
 			if ((args = malloc(size)) == NULL)
 				errx(1,"failed to allocate memory for argv structures at %zu", size);
+			memset(args, 0, size);
 			for (;; size *= 2) {
 				if ((args = realloc(args, size)) == NULL)
 					errx(1,"failed to allocate memory (size=%zu) for argv structures of pid %d", size, t.pid);
+				memset(args, 0, size);
 				mib[0] = CTL_KERN;
 				mib[1] = KERN_PROC_ARGS;
 				mib[2] = t.pid;
@@ -139,21 +139,19 @@ gboolean get_task_list (GArray *task_list)
 			free(args);
 		}
 
-		t.cpu_user = (100.0 * ((double) p.p_pctcpu / FSCALE));
-		t.cpu_system = 0; /* TODO ? */
-		/* get username from uid */
-		passwdp = getpwuid(t.uid);
-		if(passwdp != NULL && passwdp->pw_name != NULL)
-			g_strlcpy(t.uid_name, passwdp->pw_name, sizeof t.uid_name);
+		t.cpu_user = (100.0f * ((gfloat)p.p_pctcpu / FSCALE));
+		t.cpu_system = 0.0f; /* TODO ? */
 		g_array_append_val(task_list, t);
 	}
 	free(kp);
 
+	g_array_sort (task_list, task_pid_compare_fn);
+
 	return TRUE;
 }
 
 gboolean
-pid_is_sleeping (guint pid)
+pid_is_sleeping (GPid pid)
 {
 	int mib[6];
 #ifdef __OpenBSD__
@@ -206,8 +204,8 @@ gboolean get_cpu_usage (gushort *cpu_count, gfloat *cpu_user, gfloat *cpu_system
 	cur_system = cp_time[CP_SYS] + cp_time[CP_INTR];
 	cur_total = cur_user + cur_system + cp_time[CP_IDLE];
 
-	*cpu_user = (old_total > 0) ? (cur_user - old_user) * 100 / (gdouble)(cur_total - old_total) : 0;
-	*cpu_system = (old_total > 0) ? (cur_system - old_system) * 100 / (gdouble)(cur_total - old_total) : 0;
+	*cpu_user = (old_total > 0) ? (((cur_user - old_user) * 100.0f) / (float)(cur_total - old_total)) : 0.0f;
+	*cpu_system = (old_total > 0) ? (((cur_system - old_system) * 100.0f) / (float)(cur_total - old_total)) : 0.0f;
 
 	/* get #cpu */
 	size = sizeof(&cpu_count);
