@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2010  Mike Massonnet <mmassonnet@xfce.org>
  * Copyright (c) 2006  Oliver Lehmann <oliver@FreeBSD.org>
+ * Copyright (c) 2018 Rozhuk Ivan <rozhuk.im@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,10 +15,10 @@
 #include <sys/sysctl.h>
 #include <sys/user.h>
 #include <sys/proc.h>
-#include <pwd.h>
 #include <fcntl.h>
 #include <paths.h>
 #include <unistd.h>
+#include <string.h>
 #if defined(__FreeBSD_version) && __FreeBSD_version >= 900044
 #include <sys/vmmeter.h>
 #endif
@@ -26,11 +27,23 @@
 
 #include "task-manager.h"
 
-gulong
+static const gchar ki_stat2state[] = {
+	' ', /* - */
+	'R', /* SIDL */
+	'R', /* SRUN */
+	'S', /* SSLEEP */
+	'T', /* SSTOP */
+	'Z', /* SZOMB */
+	'W', /* SWAIT */
+	'L' /* SLOCK */
+};
+
+
+static guint64
 get_mem_by_bytes (const gchar *name)
 {
-	gulong buf;
-	gsize len = sizeof (gulong);
+	guint64 buf = 0;
+	gsize len = sizeof (buf);
 
 	if (sysctlbyname (name, &buf, &len, NULL, 0) < 0)
 		return 0;
@@ -38,16 +51,16 @@ get_mem_by_bytes (const gchar *name)
 	return buf;
 }
 
-guint64
+static guint64
 get_mem_by_pages (const gchar *name)
 {
-	gulong res;
+	guint64 res;
 
 	res = get_mem_by_bytes (name);
 	if (res > 0)
-		res = res * getpagesize ();
+		res *= (guint64)getpagesize ();
 
-	return (guint64) res;
+	return res;
 }
 
 gboolean
@@ -55,10 +68,10 @@ get_memory_usage (guint64 *memory_total, guint64 *memory_free, guint64 *memory_c
 {
 	/* Get memory usage */
 	{
-		*memory_total = (guint64) get_mem_by_bytes ("hw.physmem");;
+		*memory_total = get_mem_by_bytes ("hw.physmem");;
 		*memory_free = get_mem_by_pages ("vm.stats.vm.v_free_count");
 		*memory_cache = get_mem_by_pages ("vm.stats.vm.v_inactive_count");
-		*memory_buffers = (guint64) get_mem_by_bytes ("vfs.bufspace");
+		*memory_buffers = get_mem_by_bytes ("vfs.bufspace");
 	}
 
 	/* Get swap usage */
@@ -70,8 +83,8 @@ get_memory_usage (guint64 *memory_total, guint64 *memory_free, guint64 *memory_c
 			return FALSE;
 
 		kvm_getswapinfo (kd, &kswap, 1, 0);
-		*swap_total = kswap.ksw_total * getpagesize ();
-		*swap_free = (kswap.ksw_total - kswap.ksw_used) * getpagesize ();
+		*swap_total = (guint64)kswap.ksw_total * (guint64)getpagesize ();
+		*swap_free = ((guint64)(kswap.ksw_total - kswap.ksw_used)) * (guint64)getpagesize ();
 
 		kvm_close (kd);
 	}
@@ -104,11 +117,11 @@ get_cpu_usage (gushort *cpu_count, gfloat *cpu_user, gfloat *cpu_system)
 		cp_system = cpu_state[CP_SYS] + cpu_state[CP_INTR];
 		cp_total = cpu_state[CP_IDLE] + cp_user + cp_system;
 
-		*cpu_user = *cpu_system = 0.0;
+		*cpu_user = *cpu_system = 0.0f;
 		if (cp_total > cp_total_old)
 		{
-			*cpu_user = (cp_user - cp_user_old) * 100 / (gdouble)(cp_total - cp_total_old);
-			*cpu_system = (cp_system - cp_system_old) * 100 / (gdouble)(cp_total - cp_total_old);
+			*cpu_user = (((cp_user - cp_user_old) * 100.0f) / (float)(cp_total - cp_total_old));
+			*cpu_system = (((cp_system - cp_system_old) * 100.0f) / (float)(cp_total - cp_total_old));
 		}
 	}
 
@@ -116,71 +129,77 @@ get_cpu_usage (gushort *cpu_count, gfloat *cpu_user, gfloat *cpu_system)
 }
 
 static gboolean
-get_task_details (kvm_t *kd, struct kinfo_proc *kp, Task *task)
+get_task_details (struct kinfo_proc *kp, Task *task)
 {
-	struct passwd *pw;
-	char **argv;
-	int i;
+	char buf[1024], *p;
+	size_t bufsz;
+	int i, oid[4];
 
+	memset(task, 0, sizeof(Task));
 	task->pid = kp->ki_pid;
 	task->ppid = kp->ki_ppid;
-	task->cpu_user = 100 * ((double)kp->ki_pctcpu / FSCALE);
-	task->cpu_system = 0;
+	task->cpu_user = 100.0f * ((float)kp->ki_pctcpu / FSCALE);
+	task->cpu_system = 0.0f;
 	task->vsz = kp->ki_size;
-	task->rss = kp->ki_rssize * getpagesize ();
-	pw = getpwuid (kp->ki_uid);
+	task->rss = ((guint64)kp->ki_rssize * (guint64)getpagesize ());
 	task->uid = kp->ki_uid;
-	g_strlcpy (task->uid_name, (pw != NULL) ? pw->pw_name : "nobody", sizeof (task->uid_name));
-	task->prio = (gushort)kp->ki_nice;
-	g_strlcpy (task->name, kp->ki_comm, 256);
+	task->prio = (gshort)kp->ki_nice;
+	g_strlcpy (task->name, kp->ki_comm, sizeof(task->name));
 
-	task->cmdline[0] = '\0';
-	if ((argv = kvm_getargv (kd, kp, 1024)) != NULL)
-	{
-		for (i = 0; argv[i] != NULL; i++)
-		{
-			g_strlcat (task->cmdline, argv[i], 1024);
-			g_strlcat (task->cmdline, " ", 1024);
+	oid[0] = CTL_KERN;
+	oid[1] = KERN_PROC;
+	oid[2] = KERN_PROC_ARGS;
+	oid[3] = kp->ki_pid;
+	bufsz = sizeof(buf);
+	memset(buf, 0, sizeof(buf));
+	if (sysctl(oid, 4, buf, &bufsz, 0, 0) == -1) {
+		/*
+		 * If the supplied buf is too short to hold the requested
+		 * value the sysctl returns with ENOMEM. The buf is filled
+		 * with the truncated value and the returned bufsz is equal
+		 * to the requested len.
+		 */
+		if (errno != ENOMEM || bufsz != sizeof(buf)) {
+			bufsz = 0;
+		} else {
+			buf[(bufsz - 1)] = 0;
 		}
 	}
+
+	if (0 != bufsz) {
+		p = buf;
+		do {
+			g_strlcat (task->cmdline, p, sizeof(task->cmdline));
+			g_strlcat (task->cmdline, " ", sizeof(task->cmdline));
+			p += (strlen(p) + 1);
+		} while (p < buf + bufsz);
+	}
 	else
 	{
-		g_strlcpy (task->cmdline, kp->ki_comm, 1024);
+		g_strlcpy (task->cmdline, kp->ki_comm, sizeof(task->cmdline));
 	}
 
 	i = 0;
 	switch (kp->ki_stat)
 	{
+		case SIDL:
+		case SRUN:
 		case SSTOP:
-		task->state[i] = 'T';
-		break;
+		case SZOMB:
+		case SWAIT:
+		case SLOCK:
+			task->state[i] = ki_stat2state[(size_t)kp->ki_stat];
+			break;
 
 		case SSLEEP:
-		if (kp->ki_tdflags & TDF_SINTR)
-			task->state[i] = kp->ki_slptime >= MAXSLP ? 'I' : 'S';
-		else
-			task->state[i] = 'D';
-		break;
-
-		case SRUN:
-		case SIDL:
-		task->state[i] = 'R';
-		break;
-
-		case SWAIT:
-		task->state[i] = 'W';
-		break;
-
-		case SLOCK:
-		task->state[i] = 'L';
-		break;
-
-		case SZOMB:
-		task->state[i] = 'Z';
-		break;
+			if (kp->ki_tdflags & TDF_SINTR)
+				task->state[i] = kp->ki_slptime >= MAXSLP ? 'I' : 'S';
+			else
+				task->state[i] = 'D';
+			break;
 
 		default:
-		task->state[i] = '?';
+			task->state[i] = '?';
 	}
 	i++;
 	if (!(kp->ki_sflag & PS_INMEM))
@@ -203,7 +222,6 @@ get_task_details (kvm_t *kd, struct kinfo_proc *kp, Task *task)
 		task->state[i++] = '+';
 	if (kp->ki_flag & P_JAILED)
 		task->state[i++] = 'J';
-	task->state[i] = '\0';
 
 	return TRUE;
 }
@@ -213,7 +231,7 @@ get_task_list (GArray *task_list)
 {
 	kvm_t *kd;
 	struct kinfo_proc *kp;
-	int cnt, i;
+	int cnt = 0, i;
 	Task task;
 
 	if ((kd = kvm_openfiles (_PATH_DEVNULL, _PATH_DEVNULL, NULL, O_RDONLY, NULL)) == NULL)
@@ -227,17 +245,19 @@ get_task_list (GArray *task_list)
 
 	for (i = 0; i < cnt; kp++, i++)
 	{
-		get_task_details (kd, kp, &task);
+		get_task_details (kp, &task);
 		g_array_append_val (task_list, task);
 	}
 
 	kvm_close (kd);
 
+	g_array_sort (task_list, task_pid_compare_fn);
+
 	return TRUE;
 }
 
 gboolean
-pid_is_sleeping (guint pid)
+pid_is_sleeping (GPid pid)
 {
 	kvm_t *kd;
 	struct kinfo_proc *kp;
