@@ -9,7 +9,6 @@
  */
 
 #include <kstat.h>
-#include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdlib.h>
@@ -90,7 +89,7 @@ get_memory_usage (guint64 *memory_total, guint64 *memory_free, guint64 *memory_c
 }
 
 static void
-get_cpu_percent (guint pid, gulong ticks_user, gfloat *cpu_user, gulong ticks_system, gfloat *cpu_system)
+get_cpu_percent (GPid pid, gulong ticks_user, gfloat *cpu_user, gulong ticks_system, gfloat *cpu_system)
 {
 	static GHashTable *hash_cpu_user = NULL;
 	static GHashTable *hash_cpu_system = NULL;
@@ -102,22 +101,22 @@ get_cpu_percent (guint pid, gulong ticks_user, gfloat *cpu_user, gulong ticks_sy
 		hash_cpu_system = g_hash_table_new (NULL, NULL);
 	}
 
-	ticks_user_old = GPOINTER_TO_UINT (g_hash_table_lookup (hash_cpu_user, GUINT_TO_POINTER (pid)));
-	ticks_system_old = GPOINTER_TO_UINT (g_hash_table_lookup (hash_cpu_system, GUINT_TO_POINTER (pid)));
-	g_hash_table_insert (hash_cpu_user, GUINT_TO_POINTER (pid), GUINT_TO_POINTER (ticks_user));
-	g_hash_table_insert (hash_cpu_system, GUINT_TO_POINTER (pid), GUINT_TO_POINTER (ticks_system));
+	ticks_user_old = GPOINTER_TO_UINT (g_hash_table_lookup (hash_cpu_user, GINT_TO_POINTER (pid)));
+	ticks_system_old = GPOINTER_TO_UINT (g_hash_table_lookup (hash_cpu_system, GINT_TO_POINTER (pid)));
+	g_hash_table_insert (hash_cpu_user, GINT_TO_POINTER (pid), GUINT_TO_POINTER (ticks_user));
+	g_hash_table_insert (hash_cpu_system, GINT_TO_POINTER (pid), GUINT_TO_POINTER (ticks_system));
 
 	if (ticks_user < ticks_user_old || ticks_system < ticks_system_old)
 		return;
 
 	if (_cpu_count > 0 && ticks_total_delta > 0)
 	{
-		*cpu_user = (ticks_user - ticks_user_old) * 100 / (gdouble)ticks_total_delta;
-		*cpu_system = (ticks_system - ticks_system_old) * 100 / (gdouble)ticks_total_delta;
+		*cpu_user = (((ticks_user - ticks_user_old) * 100.0f) / (float)ticks_total_delta);
+		*cpu_system = (((ticks_system - ticks_system_old) * 100.0f) / (float)ticks_total_delta);
 	}
 	else
 	{
-		*cpu_user = *cpu_system = 0;
+		*cpu_user = *cpu_system = 0.0f;
 	}
 }
 
@@ -169,14 +168,13 @@ get_cpu_usage (gushort *cpu_count, gfloat *cpu_user, gfloat *cpu_system)
 }
 
 static gboolean
-get_task_details (guint pid, Task *task)
+get_task_details (GPid pid, Task *task)
 {
 	FILE *file;
 	gchar filename[96];
-	struct passwd *pw;
 	psinfo_t process;
 
-	snprintf (filename, 96, "/proc/%d/psinfo", pid);
+	snprintf (filename, sizeof(filename), "/proc/%d/psinfo", pid);
 	if ((file = fopen (filename, "r")) == NULL)
 		return FALSE;
 
@@ -186,18 +184,17 @@ get_task_details (guint pid, Task *task)
 		return FALSE;
 	}
 
-	task->pid = (guint)process.pr_pid;
-	task->ppid = (guint)process.pr_ppid;
-	g_strlcpy (task->name, process.pr_fname, 256);
-	snprintf (task->cmdline, 1024, "%s", process.pr_psargs);
-	snprintf (task->state, 16, "%c", process.pr_lwp.pr_sname);
+	memset(task, 0, sizeof(Task));
+	task->pid = process.pr_pid;
+	task->ppid = process.pr_ppid;
+	g_strlcpy (task->name, process.pr_fname, sizeof(task->name));
+	snprintf (task->cmdline, sizeof(task->cmdline), "%s", process.pr_psargs);
+	snprintf (task->state, sizeof(task->state), "%c", process.pr_lwp.pr_sname);
 	task->vsz = (guint64)process.pr_size * 1024;
 	task->rss = (guint64)process.pr_rssize * 1024;
 	task->prio = (gushort)process.pr_lwp.pr_pri;
-	pw = getpwuid (process.pr_uid);
 	task->uid = (guint)process.pr_uid;
-	g_strlcpy (task->uid_name, (pw != NULL) ? pw->pw_name : "nobody", sizeof (task->uid_name));
-	get_cpu_percent (task->pid, process.pr_time.tv_sec * 1000 + process.pr_time.tv_nsec / 100000, &task->cpu_user, 0, &task->cpu_system);
+	get_cpu_percent (task->pid, (process.pr_time.tv_sec * 1000 + process.pr_time.tv_nsec / 100000), &task->cpu_user, 0, &task->cpu_system);
 
 	fclose (file);
 
@@ -209,15 +206,15 @@ get_task_list (GArray *task_list)
 {
 	GDir *dir;
 	const gchar *name;
-	guint pid;
-	Task task = { 0 };
+	GPid pid;
+	Task task;
 
 	if ((dir = g_dir_open ("/proc", 0, NULL)) == NULL)
 		return FALSE;
 
 	while ((name = g_dir_read_name(dir)) != NULL)
 	{
-		if ((pid = (guint)g_ascii_strtoull (name, NULL, 0)) > 0)
+		if ((pid = (GPid)g_ascii_strtoull (name, NULL, 0)) > 0)
 		{
 			if (get_task_details (pid, &task))
 				g_array_append_val(task_list, task);
@@ -226,18 +223,20 @@ get_task_list (GArray *task_list)
 
 	g_dir_close (dir);
 
+	g_array_sort (task_list, task_pid_compare_fn);
+
 	return FALSE;
 }
 
 gboolean
-pid_is_sleeping (guint pid)
+pid_is_sleeping (GPid pid)
 {
 	FILE *file;
 	gchar filename[96];
 	gchar state[2];
 	psinfo_t process;
 
-	snprintf (filename, 96, "/proc/%d/psinfo", pid);
+	snprintf (filename, sizeof(filename), "/proc/%d/psinfo", pid);
 	if ((file = fopen (filename, "r")) == NULL)
 		return FALSE;
 
@@ -247,7 +246,7 @@ pid_is_sleeping (guint pid)
 		return FALSE;
 	}
 
-	snprintf (state, 2, "%c", process.pr_lwp.pr_sname);
+	snprintf (state, sizeof(state), "%c", process.pr_lwp.pr_sname);
 	fclose (file);
 
 	return (state[0] == 'T') ? TRUE : FALSE;
